/***************************************************************************
 *   Copyright (C) 2007 by Artur Szymiec                                   *
 *   artur.szymiec@gmail.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************
 *                      Orginal Code & Idea by                             *
 *                          Thomas John                                    *
 *                      thomas.john@open-design.be                         *
 *                                                                         *
 *                Linux Port & Some Minor Changes by                       *
 *                         Artur Szymiec                                   *
 *                   artur.szymiec@gmail.com	                           *
 ***************************************************************************/

#include "structures.h"
#include "camera.h"
#include <GL/gl.h>
#include <GL/glu.h>
#include <math.h>

// default constructor
camera::camera( double x_t, double y_t, double z_t, double x_v, double y_v, double z_v , double distance , struct infos_gen * infos_sz)
{
        // initialize all global variables in class

        // pointer to the structure with general info
        infos_tex_tmp = infos_sz;

        // "target" point where camera "looks at"
        infos_tex_tmp->infos_cam_sz.m_x_t = x_t;
        infos_tex_tmp->infos_cam_sz.m_y_t = y_t;
        infos_tex_tmp->infos_cam_sz.m_z_t = z_t;

        // position of the camera
        infos_tex_tmp->infos_cam_sz.m_x_v = x_v;
        infos_tex_tmp->infos_cam_sz.m_y_v = y_v;
        infos_tex_tmp->infos_cam_sz.m_z_v = z_v;

        // camera distance
        infos_tex_tmp->infos_cam_sz.m_distance = distance;

        // camera rotation
        infos_tex_tmp->infos_cam_sz.rot = 270;

        // state of the camera (1 = active)
        infos_tex_tmp->infos_cam_sz.etat = 1;

}

// destructor
camera::~camera()
{

        //nothing at the moment

}

// calculate camera position
void camera::calculer()
{
        infos_tex_tmp->infos_cam_sz.m_z_v += ((double)(infos_tex_tmp->msec) / 120.0);
        infos_tex_tmp->infos_cam_sz.m_z_t += ((double)(infos_tex_tmp->msec) / 120.0);

        // calculate camera cordinates based on the position and rotation degrees
        infos_tex_tmp->infos_cam_sz.m_x_v = (double)cos((infos_tex_tmp->infos_cam_sz.rot / 360.0) * 2.0 * 3.141) * infos_tex_tmp->infos_cam_sz.m_distance + infos_tex_tmp->infos_cam_sz.m_x_t;
        infos_tex_tmp->infos_cam_sz.m_z_v = (double)sin((infos_tex_tmp->infos_cam_sz.rot / 360.0) * 2.0 * 3.141) * infos_tex_tmp->infos_cam_sz.m_distance + infos_tex_tmp->infos_cam_sz.m_z_t;

        //inform OpenGL about the camera position
        gluLookAt ( infos_tex_tmp->infos_cam_sz.m_x_v , infos_tex_tmp->infos_cam_sz.m_y_v , infos_tex_tmp->infos_cam_sz.m_z_v , infos_tex_tmp->infos_cam_sz.m_x_t , infos_tex_tmp->infos_cam_sz.m_y_t , infos_tex_tmp->infos_cam_sz.m_z_t , 0 , 1 , 0 );
}


