/***************************************************************************
 *   Copyright (C) 2007 by Artur Szymiec                                   *
 *   artur.szymiec@gmail.com                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************
 *                      Orginal Code & Idea by                             *
 *                          Thomas John                                    *
 *                      thomas.john@open-design.be                         *
 *                                                                         *
 *                Linux Port & Some Minor Changes by                       *
 *                         Artur Szymiec                                   *
 *                   artur.szymiec@gmail.com	                           *
 ***************************************************************************/

#include "effet_particule.h"
#include <GL/gl.h>
#include <GL/glu.h>
#include <math.h>
#include "vector3f.h"
#include <cstdlib>

// initialise one particle
void effet_particule::init_particule(int particulesz)
{
	
	// cordinates of the departure
	// lets do marginal setup that the particles appear not all to the same point
	t_particules[particulesz].x = m_x + 0.5 + (double(rand()%1)-0.5) / 5;
	t_particules[particulesz].y = m_y - 1 - (double(rand()%1)-0.5) / 2;
	t_particules[particulesz].z = m_z + (double(rand()%1)-0.5) / 5;
		
	// direction of the particles
	t_particules[particulesz].d_x = (double(rand()%50)-25)/5000;
	t_particules[particulesz].d_y = -(double(rand()%3))/100; //the particle descent a little
	t_particules[particulesz].d_z = (double(rand()%50)-25)/5000;

	// speed which the particle die
	t_particules[particulesz].vitesse = 0.01 - ((double(rand()%5))/1000);

	// life (opacity)
	t_particules[particulesz].vie = 1.0;

	// size
	// was 0.7 7/10 but smaller looks better
	t_particules[particulesz].taille = 0.6 + (double(rand() % 6)/10);

	//colours
	/* SYMPA : ROUGE/ORANGE */
	t_particules[particulesz].r = 0.92;// + ((double(rand() % 10)-5)/50);
	t_particules[particulesz].g = 0.72;// + ((double(rand() % 10)-5)/50);
	t_particules[particulesz].b = 0.58;// + ((double(rand() % 10)-5)/50);
	
}

// default constructor
effet_particule::effet_particule( double x, double y, double z, double vitesse, int nombre_tot , struct infos_gen * infos_sz , int vitesse_emission , int nbr_emission )
{

	// structure that hold our all infos
	infos_tex_tmp = infos_sz;

	// put our state = 1 
	etat = 1; //(0 indicate is not active = dead)

	// initialise variables of the particle emitter position
	m_x = x;
	m_y = y;
	m_z = z;

	// speed which the particles "live" 
	// is not again utilised but not the same
	m_vitesse = vitesse;

	// max number of particles displayed - in fact time
	m_nombre_tot = nombre_tot;

	// number of particles active
	m_nombre_actifs = 0;

	// permits for specified duration emmission of the particle
	m_temps = 0;
	m_temps_tot = vitesse_emission;

	// number of the particles emmited in fasct - time 
	m_nbr_emission = nbr_emission;

	// on cre notre tableau de particules
	t_particules = new particules[nombre_tot];

}

// destrcutor
effet_particule::~effet_particule( )
{

	//

}

// calculate position  for each particle and creates one new if not exist
void effet_particule::calculer()
{

	//verifies that not exist -> create new particle
	if(m_nombre_actifs + m_nbr_emission <= m_nombre_tot && etat == 1)
	{

		m_temps += infos_tex_tmp->msec;

		if (m_temps >= m_temps_tot)
		{
			
			// create as meny as we can
			for(i=0 ; i<m_nbr_emission ; i++)
			{

				// initialize particle
				init_particule(m_nombre_actifs);
				
				m_nombre_actifs++;

			}

			// reset our counter to zero
			m_temps = 0;

		}

	}

	// puts position and opacity for each and every particle in function infos_tex_tmp->msec which is the time in ms past init display between 2 images
	for(i=0 ; i<m_nombre_actifs ; i++)
	{
		// cordinates
		t_particules[i].x += (t_particules[i].d_x * infos_tex_tmp->msec / 10);
		t_particules[i].y += (t_particules[i].d_y * infos_tex_tmp->msec / 10);
		t_particules[i].z += (t_particules[i].d_z * infos_tex_tmp->msec / 10);

		// life
		t_particules[i].vie -=  (t_particules[i].vitesse * infos_tex_tmp->msec / 10);

		// size
		t_particules[i].taille -= (t_particules[i].vitesse * infos_tex_tmp->msec / 25);

		// if life of this particle is <0 the reinit this particle
		if(t_particules[i].vie <= 0)
		{
			// reinit particle if state is 1 - enabled
			if(etat == 1)
				init_particule(i);

		}
	}

}

// display particle along with effect of type "billboarding"
// for more info about bilboarding see :  http://www.lighthouse3d.com/opengl/billboarding/index.php3?billCheat
// you'll find explanation there
void effet_particule::afficher()
{
   // recover our matrix from GL
    glGetFloatv( GL_MODELVIEW_MATRIX, mat );

	// declare vectors which will allow us calculate bilboarding
    vRight.x=mat[0];
    vRight.y=mat[4];
    vRight.z=mat[8];
    vUp.x=mat[1];
    vUp.y=mat[5];
    vUp.z=mat[9];
    
    //double fAdjustedSize = 0.4 + (double(rand() % 4)/10);
	
	glBegin(GL_QUADS);

	// display all the active particles
	for(int i = 0 ; i<m_nombre_actifs ; i++)
	{		
		// if particle is "live"
		if(t_particules[i].vie > 0)
		{

			// define centre
			vCenter.x = t_particules[i].x;
			vCenter.y = t_particules[i].y;
			vCenter.z = t_particules[i].z;

			// calculate cordinates od 4 points that are a part of particle
			vPoint0 = vCenter + ((-vRight - vUp) * t_particules[i].taille );
			vPoint1 = vCenter + (( vRight - vUp) * t_particules[i].taille );
			vPoint2 = vCenter + (( vRight + vUp) * t_particules[i].taille );
			vPoint3 = vCenter + ((-vRight + vUp) * t_particules[i].taille );
			
			// adjust a colour of the particle (opacity depends on the mode of blending)
			//glColor3d( t_particules[i].vie / 4 , t_particules[i].vie / 1.1, t_particules[i].vie / 1.5 );
			glColor3d( t_particules[i].vie * t_particules[i].r , t_particules[i].vie * t_particules[i].g, t_particules[i].vie * t_particules[i].b );

			// draw the particle
			glTexCoord2d( 0.0, 0.0 );
			glVertex3d( vPoint0.x, vPoint0.y, vPoint0.z );

			glTexCoord2d( 1.0, 0.0 );
			glVertex3d( vPoint1.x, vPoint1.y, vPoint1.z );

			glTexCoord2d( 1.0, 1.0);
			glVertex3d( vPoint2.x, vPoint2.y, vPoint2.z );

			glTexCoord2d( 0.0, 1.0 );
			glVertex3d( vPoint3.x, vPoint3.y, vPoint3.z );

		}
	}

	glEnd();
}

